package fr.username404.snowygui.fabric

import com.mojang.blaze3d.vertex.PoseStack
import fr.username404.snowygui.EventSnowy
import fr.username404.snowygui.Snowy
import fr.username404.snowygui.gui.feature.ButtonImpl
import fr.username404.snowygui.gui.feature.Zoom
import net.fabricmc.api.ClientModInitializer
import net.fabricmc.fabric.api.client.rendering.v1.HudRenderCallback
import net.fabricmc.fabric.api.client.rendering.v1.HudRenderCallback.EVENT
import net.fabricmc.loader.api.FabricLoader
import net.fabricmc.loader.launch.common.FabricLauncherBase
import kotlin.io.path.exists
import kotlin.io.path.isDirectory
import kotlin.io.path.listDirectoryEntries
import kotlin.io.path.pathString

class FabricInit: Snowy(), ClientModInitializer {
    override fun onInitializeClient() {
        atInit()
        EVENT.register(
            object: HudRenderCallback, EventSnowy {
                override val type: String = "HudRender"
                override fun onHudRender(matrixStack: PoseStack?, tickDelta: Float) = fire(matrixStack)
            }
        )
    }
    init {
        val modsPaths = FabricLoader.getInstance().allMods.map {
            it.getPath(FeaturePackage.replace('.', '/'))
        }.filter { it.exists() && it.isDirectory() }
        annotatedButtons = modsPaths.flatMap { buttonsDirectory ->
            val classSet = mutableSetOf<Class<out ButtonImpl>>()
            buttonsDirectory.run {
                listDirectoryEntries("*.class").forEach { file ->
                    val foundClass = FabricLauncherBase.getClass(file.pathString.drop(1).replace('/', '.').removeSuffix(".class"))
                    if (foundClass.isValidForButtonCollection()) classSet.add(foundClass.asSubclass(ButtonImpl::class.java))
                }
            }
            classSet
        }.toSet()
    }
}

@JvmField
var isOkZoomerPresent: Boolean = FabricLoader.getInstance().isModLoaded("okzoomer")
private val okZoomerPairs by lazy {
    try {
        with(Class.forName("io.github.joaoh1.okzoomer.client.utils.ZoomUtils")) {
            (getField("zoomDivisor") to null) to (getField("zoomState") to null)
        }
    } catch (e: ClassNotFoundException) {
        with(Class.forName("io.github.ennuil.okzoomer.utils.ZoomUtils").getDeclaredField("zoomerZoom")) {
            get(null).javaClass.run {
                (getDeclaredField("zoomDivisor") to this@with.get(null)) to (getDeclaredField("zoom") to this@with.get(null))
            }
        }
    }.apply { first.first.isAccessible = true; second.first.isAccessible = true; }
}

fun fabricZoom() {
    try {
        with(okZoomerPairs) {
            first.run { first.setDouble(second, Zoom.zoomFactor) }
            second.run { first.setBoolean(second, Zoom.toggled) }
        }
    } catch (e: Exception) {
        isOkZoomerPresent = false
    }
}